@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Device for tracking bounding box */
@@ -24,11 +24,13 @@
 #include "gdevbbox.h"
 #include "gxdcolor.h"		/* for gx_device_black/white */
 #include "gxiparam.h"		/* for image source size */
-#include "gxistate.h"
+#include "gxgstate.h"
 #include "gxpaint.h"
 #include "gxpath.h"
 #include "gxcpath.h"
 
+#include "gdevkrnlsclass.h" /* 'standard' built in subclasses, currently First/Last Page and object filter */
+
 /* GC descriptor */
 public_st_device_bbox();
 
@@ -50,14 +52,45 @@ static dev_proc_fill_parallelogram(bbox_fill_parallelogram);
 static dev_proc_fill_triangle(bbox_fill_triangle);
 static dev_proc_draw_thin_line(bbox_draw_thin_line);
 static dev_proc_strip_tile_rectangle(bbox_strip_tile_rectangle);
-static dev_proc_strip_copy_rop(bbox_strip_copy_rop);
 static dev_proc_strip_copy_rop2(bbox_strip_copy_rop2);
 static dev_proc_strip_tile_rect_devn(bbox_strip_tile_rect_devn);
 static dev_proc_begin_typed_image(bbox_begin_typed_image);
-static dev_proc_create_compositor(bbox_create_compositor);
+static dev_proc_composite(bbox_composite);
 static dev_proc_text_begin(bbox_text_begin);
 static dev_proc_fillpage(bbox_fillpage);
 
+static void
+bbox_initialize_device_procs(gx_device *dev)
+{
+     set_dev_proc(dev, open_device, bbox_open_device);
+     set_dev_proc(dev, get_initial_matrix, gx_upright_get_initial_matrix);
+     set_dev_proc(dev, output_page, bbox_output_page);
+     set_dev_proc(dev, close_device, bbox_close_device);
+     set_dev_proc(dev, map_rgb_color, gx_default_gray_map_rgb_color);
+     set_dev_proc(dev, map_color_rgb, gx_default_gray_map_color_rgb);
+     set_dev_proc(dev, fill_rectangle, bbox_fill_rectangle);
+     set_dev_proc(dev, copy_mono, bbox_copy_mono);
+     set_dev_proc(dev, copy_color, bbox_copy_color);
+     set_dev_proc(dev, get_params, bbox_get_params);
+     set_dev_proc(dev, put_params, bbox_put_params);
+     set_dev_proc(dev, get_page_device, gx_page_device_get_page_device);
+     set_dev_proc(dev, copy_alpha, bbox_copy_alpha);
+     set_dev_proc(dev, fill_path, bbox_fill_path);
+     set_dev_proc(dev, stroke_path, bbox_stroke_path);
+     set_dev_proc(dev, fill_mask, bbox_fill_mask);
+     set_dev_proc(dev, fill_trapezoid, bbox_fill_trapezoid);
+     set_dev_proc(dev, fill_parallelogram, bbox_fill_parallelogram);
+     set_dev_proc(dev, fill_triangle, bbox_fill_triangle);
+     set_dev_proc(dev, draw_thin_line, bbox_draw_thin_line);
+     set_dev_proc(dev, strip_tile_rectangle, bbox_strip_tile_rectangle);
+     set_dev_proc(dev, begin_typed_image, bbox_begin_typed_image);
+     set_dev_proc(dev, composite, bbox_composite);
+     set_dev_proc(dev, text_begin, bbox_text_begin);
+     set_dev_proc(dev, fillpage, bbox_fillpage);
+     set_dev_proc(dev, strip_copy_rop2, bbox_strip_copy_rop2);
+     set_dev_proc(dev, strip_tile_rect_devn, bbox_strip_tile_rect_devn);
+}
+
 /* The device prototype */
 /*
  * Normally this would be static, but if the device is going to be used
@@ -83,82 +116,11 @@ gx_device_bbox gs_bbox_device =
     /*
      * Define the device as 8-bit gray scale to avoid computing halftones.
      */
-    std_device_dci_body(gx_device_bbox, 0, "bbox",
+    std_device_dci_body(gx_device_bbox, bbox_initialize_device_procs, "bbox",
                         MAX_COORD, MAX_COORD,
                         MAX_RESOLUTION, MAX_RESOLUTION,
                         1, 8, 255, 0, 256, 1),
-    {bbox_open_device,
-     gx_upright_get_initial_matrix,
-     NULL,			/* sync_output */
-     bbox_output_page,
-     bbox_close_device,
-     gx_default_gray_map_rgb_color,
-     gx_default_gray_map_color_rgb,
-     bbox_fill_rectangle,
-     NULL,			/* tile_rectangle */
-     bbox_copy_mono,
-     bbox_copy_color,
-     NULL,			/* draw_line */
-     NULL,			/* get_bits */
-     bbox_get_params,
-     bbox_put_params,
-     gx_default_map_cmyk_color,
-     NULL,			/* get_xfont_procs */
-     NULL,			/* get_xfont_device */
-     gx_default_map_rgb_alpha_color,
-     gx_page_device_get_page_device,
-     NULL,			/* get_alpha_bits */
-     bbox_copy_alpha,
-     NULL,			/* get_band */
-     NULL,			/* copy_rop */
-     bbox_fill_path,
-     bbox_stroke_path,
-     bbox_fill_mask,
-     bbox_fill_trapezoid,
-     bbox_fill_parallelogram,
-     bbox_fill_triangle,
-     bbox_draw_thin_line,
-     gx_default_begin_image,
-     NULL,			/* image_data */
-     NULL,			/* end_image */
-     bbox_strip_tile_rectangle,
-     bbox_strip_copy_rop,
-     NULL,			/* get_clipping_box */
-     bbox_begin_typed_image,
-     NULL,			/* get_bits_rectangle */
-     gx_default_map_color_rgb_alpha,
-     bbox_create_compositor,
-     NULL,			/* get_hardware_params */
-     bbox_text_begin,
-     NULL,			/* finish_copydevice */
-     NULL,			/* begin_transparency_group */
-     NULL,			/* end_transparency_group */
-     NULL,			/* begin_transparency_mask */
-     NULL,			/* end_transparency_mask */
-     NULL,			/* discard_transparency_layer */
-     NULL,			/* get_color_mapping_procs */
-     NULL,			/* get_color_comp_index */
-     NULL,			/* encode_color */
-     NULL,			/* decode_color */
-     NULL,			/* pattern_manage */
-     NULL,			/* fill_rectangle_hl_color */
-     NULL,			/* include_color_space */
-     NULL,			/* fill_linear_color_scanline */
-     NULL,			/* fill_linear_color_trapezoid */
-     NULL,			/* fill_linear_color_triangle */
-     NULL,			/* update_spot_equivalent_colors */
-     NULL,			/* ret_devn_params */
-     bbox_fillpage,		/* fillpage */
-     NULL,                      /* push_transparency_state */
-     NULL,                      /* pop_transparency_state */
-     NULL,                      /* put_image */
-     NULL,                      /* dev_spec_op */
-     NULL,                      /* copy_planes */
-     NULL,                      /* get_profile */
-     NULL,                      /* set_graphics_type_tag */
-     bbox_strip_copy_rop2,
-     bbox_strip_tile_rect_devn
-    },
+    { 0 },
     0,				/* target */
     1,				/*true *//* free_standing */
     1				/*true *//* forward_open_close */
@@ -273,15 +235,15 @@ bbox_close_device(gx_device * dev)
 void
 gx_device_bbox_init(gx_device_bbox * dev, gx_device * target, gs_memory_t *mem)
 {
-    gx_device_init((gx_device *) dev, (const gx_device *)&gs_bbox_device,
-                   (target ? target->memory : mem), true);
+    /* Can never fail */
+    (void)gx_device_init((gx_device *) dev, (const gx_device *)&gs_bbox_device,
+                         (target ? target->memory : mem), true);
     if (target) {
         gx_device_forward_fill_in_procs((gx_device_forward *) dev);
         set_dev_proc(dev, get_initial_matrix, gx_forward_get_initial_matrix);
         set_dev_proc(dev, map_rgb_color, gx_forward_map_rgb_color);
         set_dev_proc(dev, map_color_rgb, gx_forward_map_color_rgb);
         set_dev_proc(dev, map_cmyk_color, gx_forward_map_cmyk_color);
-        set_dev_proc(dev, map_rgb_alpha_color, gx_forward_map_rgb_alpha_color);
         set_dev_proc(dev, get_color_mapping_procs, gx_forward_get_color_mapping_procs);
         set_dev_proc(dev, get_color_comp_index, gx_forward_get_color_comp_index);
         set_dev_proc(dev, encode_color, gx_forward_encode_color);
@@ -329,10 +291,11 @@ gx_device_bbox_release(gx_device_bbox *dev)
 }
 
 /* Read back the bounding box in 1/72" units. */
-void
+int
 gx_device_bbox_bbox(gx_device_bbox * dev, gs_rect * pbbox)
 {
     gs_fixed_rect bbox;
+    int code;
 
     BBOX_GET_BOX(dev, &bbox);
     if (bbox.p.x > bbox.q.x || bbox.p.y > bbox.q.y) {
@@ -347,19 +310,27 @@ gx_device_bbox_bbox(gx_device_bbox * dev, gs_rect * pbbox)
         dbox.q.x = fixed2float(bbox.q.x);
         dbox.q.y = fixed2float(bbox.q.y);
         gs_deviceinitialmatrix((gx_device *)dev, &mat);
-        gs_bbox_transform_inverse(&dbox, &mat, pbbox);
+        code = gs_bbox_transform_inverse(&dbox, &mat, pbbox);
+        if (code < 0)
+            return code;
     }
+    return 0;
 }
 
 static int
 bbox_open_device(gx_device * dev)
 {
-    gx_device_bbox *const bdev = (gx_device_bbox *) dev;
+    gx_device_bbox *bdev = (gx_device_bbox *) dev;
+    int code;
 
     if (bdev->free_standing) {
         gx_device_forward_fill_in_procs((gx_device_forward *) dev);
         bdev->box_procs = box_procs_default;
         bdev->box_proc_data = bdev;
+
+        code = install_internal_subclass_devices((gx_device **)&bdev, NULL);
+        if (code < 0)
+            return code;
     }
     if (bdev->box_procs.init_box == box_procs_default.init_box)
         BBOX_INIT_BOX(bdev);
@@ -384,8 +355,11 @@ bbox_output_page(gx_device * dev, int num_copies, int flush)
          * This is a free-standing device.  Print the page bounding box.
          */
         gs_rect bbox;
+        int code;
 
-        gx_device_bbox_bbox(bdev, &bbox);
+        code = gx_device_bbox_bbox(bdev, &bbox);
+        if (code < 0)
+            return code;
         dmlprintf4(dev->memory, "%%%%BoundingBox: %d %d %d %d\n",
                    (int)floor(bbox.p.x), (int)floor(bbox.p.y),
                    (int)ceil(bbox.q.x), (int)ceil(bbox.q.y));
@@ -403,7 +377,8 @@ bbox_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
     gx_device *tdev = bdev->target;
-    /* gx_forward_fill_rectangle doesn't exist */
+    /* gx_forward_fill_rectangle exists, but does the wrong thing in
+     * the event of a NULL target, so open code it here. */
     int code =
         (tdev == 0 ? 0 :
          dev_proc(tdev, fill_rectangle)(tdev, x, y, w, h, color));
@@ -418,7 +393,8 @@ bbox_copy_mono(gx_device * dev, const byte * data,
                gx_color_index zero, gx_color_index one)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
-    /* gx_forward_copy_mono doesn't exist */
+    /* gx_forward_copy_mono exists, but does the wrong thing in
+     * the event of a NULL target, so open code it here. */
     gx_device *tdev = bdev->target;
     int code =
         (tdev == 0 ? 0 :
@@ -437,7 +413,8 @@ bbox_copy_color(gx_device * dev, const byte * data,
             int dx, int raster, gx_bitmap_id id, int x, int y, int w, int h)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
-    /* gx_forward_copy_color doesn't exist */
+    /* gx_forward_copy_color exists, but does the wrong thing in
+     * the event of a NULL target, so open code it here. */
     gx_device *tdev = bdev->target;
     int code =
         (tdev == 0 ? 0 :
@@ -454,7 +431,8 @@ bbox_copy_alpha(gx_device * dev, const byte * data, int data_x,
                 gx_color_index color, int depth)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
-    /* gx_forward_copy_alpha doesn't exist */
+    /* gx_forward_copy_alpha exists, but does the wrong thing in
+     * the event of a NULL target, so open code it here. */
     gx_device *tdev = bdev->target;
     int code =
         (tdev == 0 ? 0 :
@@ -483,7 +461,7 @@ bbox_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
 
 static int
 bbox_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tiles,
-   int x, int y, int w, int h, const gx_drawing_color *pdcolor0, 
+   int x, int y, int w, int h, const gx_drawing_color *pdcolor0,
    const gx_drawing_color *pdcolor1, int px, int py)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
@@ -497,29 +475,6 @@ bbox_strip_tile_rect_devn(gx_device * dev, const gx_strip_bitmap * tiles,
     return code;
 }
 
-static int
-bbox_strip_copy_rop(gx_device * dev,
-                    const byte * sdata, int sourcex, uint sraster,
-                    gx_bitmap_id id,
-                    const gx_color_index * scolors,
-                    const gx_strip_bitmap * textures,
-                    const gx_color_index * tcolors,
-                    int x, int y, int w, int h,
-                    int phase_x, int phase_y, gs_logical_operation_t lop)
-{
-    gx_device_bbox *const bdev = (gx_device_bbox *) dev;
-    /* gx_forward_strip_copy_rop doesn't exist */
-    gx_device *tdev = bdev->target;
-    int code =
-        (tdev == 0 ? 0 :
-         dev_proc(tdev, strip_copy_rop)
-         (tdev, sdata, sourcex, sraster, id, scolors,
-          textures, tcolors, x, y, w, h, phase_x, phase_y, lop));
-
-    BBOX_ADD_INT_RECT(bdev, x, y, x + w, y + h);
-    return code;
-}
-
 static int
 bbox_strip_copy_rop2(gx_device * dev,
                     const byte * sdata, int sourcex, uint sraster,
@@ -532,7 +487,8 @@ bbox_strip_copy_rop2(gx_device * dev,
                     uint planar_height)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
-    /* gx_forward_strip_copy_rop doesn't exist */
+    /* gx_forward_strip_copy_rop2 exists, but does the wrong thing in
+     * the event of a NULL target, so open code it here. */
     gx_device *tdev = bdev->target;
     int code =
         (tdev == 0 ? 0 :
@@ -604,6 +560,7 @@ bbox_put_params(gx_device * dev, gs_param_list * plist)
         default:
             ecode = code;
             e:param_signal_error(plist, param_name, ecode);
+            /* fall through */
         case 1:
             bba.data = 0;
     }
@@ -795,16 +752,16 @@ bbox_draw_thin_line(gx_device * dev,
  (pbox)->q.x += (adj).x, (pbox)->q.y += (adj).y)
 
 static int
-bbox_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
+bbox_fill_path(gx_device * dev, const gs_gstate * pgs, gx_path * ppath,
                const gx_fill_params * params, const gx_device_color * pdevc,
                const gx_clip_path * pcpath)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
     gx_device *tdev = bdev->target;
     dev_proc_fill_path((*fill_path)) =
-        (tdev == 0 ? dev_proc(&gs_null_device, fill_path) :
-         dev_proc(tdev, fill_path));
+        (tdev == 0 ? NULL : dev_proc(tdev, fill_path));
     int code;
+    gx_drawing_color devc;
 
     if (ppath == NULL) {
         /* A special handling of shfill with no path. */
@@ -830,42 +787,48 @@ bbox_fill_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
          * If the path lies within the already accumulated box, just draw
          * on the target.
          */
-        if (BBOX_IN_RECT(bdev, &ibox))
-            return fill_path(tdev, pis, ppath, params, pdevc, pcpath);
-        /*
-         * If the target uses the default algorithm, just draw on the
-         * bbox device.
-         */
-        if (tdev != 0 && fill_path == gx_default_fill_path)
-            return fill_path(dev, pis, ppath, params, pdevc, pcpath);
-        /* Draw on the target now. */
-        code = fill_path(tdev, pis, ppath, params, pdevc, pcpath);
-        if (code < 0)
-            return code;
-        if (pcpath != NULL &&
-            !gx_cpath_includes_rectangle(pcpath, ibox.p.x, ibox.p.y,
-                                         ibox.q.x, ibox.q.y)
-            ) {
-            /*
-             * Let the target do the drawing, but break down the
-             * fill path into pieces for computing the bounding box.
-             */
-            gx_drawing_color devc;
-
-            set_nonclient_dev_color(&devc, bdev->black);  /* any non-white color will do */
-            bdev->target = NULL;
-            code = gx_default_fill_path(dev, pis, ppath, params, &devc, pcpath);
-            bdev->target = tdev;
-        } else {		/* Just use the path bounding box. */
-            BBOX_ADD_RECT(bdev, ibox.p.x, ibox.p.y, ibox.q.x, ibox.q.y);
+        if (BBOX_IN_RECT(bdev, &ibox)) {
+            /* If we have no target device, just exit */
+            if (fill_path == NULL)
+                return 0;
+            return fill_path(tdev, pgs, ppath, params, pdevc, pcpath);
         }
+        if (tdev != 0) {
+            /*
+             * If the target uses the default algorithm, just draw on the
+             * bbox device.
+             */
+            if (fill_path == gx_default_fill_path)
+                return fill_path(dev, pgs, ppath, params, pdevc, pcpath);
+            /* Draw on the target now. */
+            code = fill_path(tdev, pgs, ppath, params, pdevc, pcpath);
+            if (code < 0)
+                return code;
+        }
+
+        /* Previously we would use the path bbox above usually, but that bbox is
+         * inaccurate for curves, because it considers the control points of the
+         * curves to be included whcih of course they are not. Now we scan-convert
+         * the path to get an accurate result, just as we do for strokes.
+         */
+        /*
+         * Draw the path, but break down the
+         * fill path into pieces for computing the bounding box accurately.
+         */
+
+        set_nonclient_dev_color(&devc, bdev->black);  /* any non-white color will do */
+        bdev->target = NULL;
+        code = gx_default_fill_path(dev, pgs, ppath, params, &devc, pcpath);
+        bdev->target = tdev;
         return code;
-    } else
-        return fill_path(tdev, pis, ppath, params, pdevc, pcpath);
+    } else if (fill_path == NULL)
+            return 0;
+    else
+        return fill_path(tdev, pgs, ppath, params, pdevc, pcpath);
 }
 
 static int
-bbox_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
+bbox_stroke_path(gx_device * dev, const gs_gstate * pgs, gx_path * ppath,
                  const gx_stroke_params * params,
                  const gx_drawing_color * pdevc, const gx_clip_path * pcpath)
 {
@@ -874,41 +837,24 @@ bbox_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
     /* Skip the call if there is no target. */
     int code =
         (tdev == 0 ? 0 :
-         dev_proc(tdev, stroke_path)(tdev, pis, ppath, params, pdevc, pcpath));
+         dev_proc(tdev, stroke_path)(tdev, pgs, ppath, params, pdevc, pcpath));
 
     if (!GX_DC_IS_TRANSPARENT(pdevc, bdev)) {
         gs_fixed_rect ibox;
         gs_fixed_point expand;
+        int ibox_valid = 0;
 
-        if (gx_stroke_path_expansion(pis, ppath, &expand) == 0 &&
+        if (gx_stroke_path_expansion(pgs, ppath, &expand) == 0 &&
             gx_path_bbox(ppath, &ibox) >= 0
             ) {
             /* The fast result is exact. */
             adjust_box(&ibox, expand);
-        } else {
-            /*
-             * The result is not exact.  Compute an exact result using
-             * strokepath.
-             */
-            gx_path *spath = gx_path_alloc(pis->memory, "bbox_stroke_path");
-            int code = 0;
-
-            if (spath)
-                code = gx_imager_stroke_add(ppath, spath, dev, pis);
-            else
-                code = -1;
-            if (code >= 0)
-                code = gx_path_bbox(spath, &ibox);
-            if (code < 0) {
-                ibox.p.x = ibox.p.y = min_fixed;
-                ibox.q.x = ibox.q.y = max_fixed;
-            }
-            if (spath)
-                gx_path_free(spath, "bbox_stroke_path");
+            ibox_valid = 1;
         }
-        if (pcpath != NULL &&
-            !gx_cpath_includes_rectangle(pcpath, ibox.p.x, ibox.p.y,
-                                         ibox.q.x, ibox.q.y)
+        if (!ibox_valid ||
+            (pcpath != NULL &&
+             !gx_cpath_includes_rectangle(pcpath, ibox.p.x, ibox.p.y,
+                                         ibox.q.x, ibox.q.y))
             ) {
             /* Let the target do the drawing, but break down the */
             /* fill path into pieces for computing the bounding box. */
@@ -916,7 +862,7 @@ bbox_stroke_path(gx_device * dev, const gs_imager_state * pis, gx_path * ppath,
 
             set_nonclient_dev_color(&devc, bdev->black);  /* any non-white color will do */
             bdev->target = NULL;
-            gx_default_stroke_path(dev, pis, ppath, params, &devc, pcpath);
+            gx_default_stroke_path(dev, pgs, ppath, params, &devc, pcpath);
             bdev->target = tdev;
         } else {
             /* Just use the path bounding box. */
@@ -954,8 +900,9 @@ bbox_fill_mask(gx_device * dev,
                              pdcolor, depth, lop, pcpath);
         bdev->target = tdev;
     } else {
-        /* Just use the mask bounding box. */
-        BBOX_ADD_INT_RECT(bdev, x, y, x + w, y + h);
+        if (w > 0 && h > 0)
+            /* Just use the mask bounding box. */
+            BBOX_ADD_INT_RECT(bdev, x, y, x + w, y + h);
     }
     return code;
 }
@@ -986,7 +933,7 @@ static const gx_image_enum_procs_t bbox_image_enum_procs = {
 };
 
 static int
-bbox_image_begin(const gs_imager_state * pis, const gs_matrix * pmat,
+bbox_image_begin(const gs_gstate * pgs, const gs_matrix * pmat,
                  const gs_image_common_t * pic, const gs_int_rect * prect,
                  const gx_clip_path * pcpath, gs_memory_t * memory,
                  bbox_image_enum ** ppbe)
@@ -996,7 +943,7 @@ bbox_image_begin(const gs_imager_state * pis, const gs_matrix * pmat,
     bbox_image_enum *pbe;
 
     if (pmat == 0)
-        pmat = &ctm_only(pis);
+        pmat = &ctm_only(pgs);
     if ((code = gs_matrix_invert(&pic->ImageMatrix, &mat)) < 0 ||
         (code = gs_matrix_multiply(&mat, pmat, &mat)) < 0
         )
@@ -1014,15 +961,8 @@ bbox_image_begin(const gs_imager_state * pis, const gs_matrix * pmat,
         pbe->x0 = prect->p.x, pbe->x1 = prect->q.x;
         pbe->y = prect->p.y, pbe->height = prect->q.y - prect->p.y;
     } else {
-        gs_int_point size;
-        int code = (*pic->type->source_size) (pis, pic, &size);
-
-        if (code < 0) {
-            gs_free_object(memory, pbe, "bbox_image_begin");
-            return code;
-        }
-        pbe->x0 = 0, pbe->x1 = size.x;
-        pbe->y = 0, pbe->height = size.y;
+        pbe->x0 = 0, pbe->x1 = pic->Width;
+        pbe->y = 0, pbe->height = pic->Height;
     }
     *ppbe = pbe;
     return 0;
@@ -1042,7 +982,7 @@ bbox_image_copy_target_info(bbox_image_enum * pbe)
 
 static int
 bbox_begin_typed_image(gx_device * dev,
-                       const gs_imager_state * pis, const gs_matrix * pmat,
+                       const gs_gstate * pgs, const gs_matrix * pmat,
                    const gs_image_common_t * pic, const gs_int_rect * prect,
                        const gx_drawing_color * pdcolor,
                        const gx_clip_path * pcpath,
@@ -1050,12 +990,12 @@ bbox_begin_typed_image(gx_device * dev,
 {
     bbox_image_enum *pbe;
     int code =
-        bbox_image_begin(pis, pmat, pic, prect, pcpath, memory, &pbe);
+        bbox_image_begin(pgs, pmat, pic, prect, pcpath, memory, &pbe);
 
     if (code < 0)
         return code;
     /*
-     * If there is no target, we still have to call default_begin_image
+     * If there is no target, we still have to call default_begin_typed_image
      * to get the correct num_planes and plane_depths.
      */
     {
@@ -1071,7 +1011,7 @@ bbox_begin_typed_image(gx_device * dev,
             begin_typed_image = dev_proc(tdev, begin_typed_image);
         }
         code = (*begin_typed_image)
-            (tdev, pis, pmat, pic, prect, pdcolor, pcpath, memory,
+            (tdev, pgs, pmat, pic, prect, pdcolor, pcpath, memory,
              &pbe->target_info);
         if (code) {
             bbox_image_end_image((gx_image_enum_common_t *)pbe, false);
@@ -1149,8 +1089,9 @@ bbox_image_plane_data(gx_image_enum_common_t * info,
                                  bx2, by2, &devc, lop_default);
         bdev->target = tdev;
     } else {
-        /* Just use the bounding box. */
-        BBOX_ADD_RECT(bdev, ibox.p.x, ibox.p.y, ibox.q.x, ibox.q.y);
+        /* Just use the bounding box if the image is not 0 width or height */
+        if (ibox.p.x != ibox.q.x && ibox.p.y != ibox.q.y)
+            BBOX_ADD_RECT(bdev, ibox.p.x, ibox.p.y, ibox.q.x, ibox.q.y);
     }
     return code;
 }
@@ -1222,9 +1163,9 @@ static const gx_device_bbox_procs_t box_procs_forward = {
 };
 
 static int
-bbox_create_compositor(gx_device * dev,
+bbox_composite(gx_device * dev,
                        gx_device ** pcdev, const gs_composite_t * pcte,
-                       gs_imager_state * pis, gs_memory_t * memory, gx_device *cindev)
+                       gs_gstate * pgs, gs_memory_t * memory, gx_device *cindev)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
     gx_device *target = bdev->target;
@@ -1245,17 +1186,17 @@ bbox_create_compositor(gx_device * dev,
     {
         gx_device *temp_cdev;
         gx_device_bbox *bbcdev;
-        int code = (*dev_proc(target, create_compositor))
-            (target, &temp_cdev, pcte, pis, memory, cindev);
+        int code = (*dev_proc(target, composite))
+            (target, &temp_cdev, pcte, pgs, memory, cindev);
 
         /* If the target did not create a new compositor then we are done. */
-        if (code < 0 || target == temp_cdev) {
+        if (code <= 0) {
             *pcdev = dev;
             return code;
         }
         bbcdev = gs_alloc_struct_immovable(memory, gx_device_bbox,
                                            &st_device_bbox,
-                                           "bbox_create_compositor");
+                                           "bbox_composite");
         if (bbcdev == 0) {
             (*dev_proc(temp_cdev, close_device)) (temp_cdev);
             return_error(gs_error_VMerror);
@@ -1265,22 +1206,23 @@ bbox_create_compositor(gx_device * dev,
         bbcdev->box_procs = box_procs_forward;
         bbcdev->box_proc_data = bdev;
         *pcdev = (gx_device *) bbcdev;
-        return 0;
+        /* We return 1 to indicate that a new compositor was created
+         * that wrapped dev. */
+        return 1;
     }
 }
 
 /* ------ Text imaging ------ */
 
 static int
-bbox_text_begin(gx_device * dev, gs_imager_state * pis,
+bbox_text_begin(gx_device * dev, gs_gstate * pgs,
                 const gs_text_params_t * text, gs_font * font,
-                gx_path * path, const gx_device_color * pdcolor,
                 const gx_clip_path * pcpath,
-                gs_memory_t * memory, gs_text_enum_t ** ppenum)
+                gs_text_enum_t ** ppenum)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
-    int code = gx_default_text_begin(dev, pis, text, font, path, pdcolor,
-                                     pcpath, memory, ppenum);
+    int code = gx_default_text_begin(dev, pgs, text, font,
+                                     pcpath, ppenum);
 
     if (code >=0 && bdev->target != NULL) {
         /* See note on imaging_dev in gxtext.h */
@@ -1292,7 +1234,7 @@ bbox_text_begin(gx_device * dev, gs_imager_state * pis,
 
 /* --------------- fillpage ------------------- */
 
-int bbox_fillpage(gx_device *dev, gs_imager_state * pis, gx_device_color *pdevc)
+int bbox_fillpage(gx_device *dev, gs_gstate * pgs, gx_device_color *pdevc)
 {
     /* Call the target's proc, but don't account the size. */
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
@@ -1301,5 +1243,5 @@ int bbox_fillpage(gx_device *dev, gs_imager_state * pis, gx_device_color *pdevc)
     BBOX_INIT_BOX(bdev);
     if (tdev == NULL)
         return 0;
-    return dev_proc(tdev, fillpage)(tdev, pis, pdevc);
+    return dev_proc(tdev, fillpage)(tdev, pgs, pdevc);
 }
