@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2022 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -511,6 +511,17 @@ enum {
      *  Returns >0 if replacement occurred. */
     gxdso_replacecolor,
 
+    /* Used solely (for now at least) by pdfwrite. We need to tell pdfwrite that Image and Form
+     * XObjects, have an /OC entry and what the object number of the optional content entry is.
+     * We should already have defined the optional content dictionary using a pdfmark.
+     */
+    gxdso_pending_optional_content,
+
+    /* Called to give a device the chance to update the color information after put_params
+     * has reset it. This enables device specific changes to not interfere when devices are
+     * cloned for NumRenderingThreads etc. */
+    gxdso_adjust_colors,
+
     /* Add new gxdso_ keys above this. */
     gxdso_pattern__LAST
 };
