@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2021 Artifex Software, Inc.
+/* Copyright (C) 2001-2024 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 
@@ -27,88 +27,14 @@
 #ifdef PDF_DEVICE_NAME
 
 const gx_device_pdf PDF_DEVICE_IDENT =
-{std_device_dci_type_body(gx_device_pdf, 0, PDF_DEVICE_NAME,
+{std_device_dci_type_body(gx_device_pdf, pdfwrite_initialize_device_procs,
+                          PDF_DEVICE_NAME,
                           &st_device_pdfwrite,
                           DEFAULT_WIDTH_10THS * X_DPI / 10,
                           DEFAULT_HEIGHT_10THS * Y_DPI / 10,
                           X_DPI, Y_DPI,
                           3, 24, 255, 255, 256, 256),
- {pdf_open,
-  gx_upright_get_initial_matrix,
-  NULL,				/* sync_output */
-  pdf_output_page,
-  pdf_close,
-  gx_default_rgb_map_rgb_color,
-  gx_default_rgb_map_color_rgb,
-  gdev_pdf_fill_rectangle,
-  NULL,				/* tile_rectangle */
-  gdev_pdf_copy_mono,
-  gdev_pdf_copy_color,
-  NULL,				/* draw_line */
-  psdf_get_bits,		/* get_bits */
-  gdev_pdf_get_params,
-  gdev_pdf_put_params,
-  NULL,				/* map_cmyk_color */
-  NULL,				/* get_xfont_procs */
-  NULL,				/* get_xfont_device */
-  NULL,				/* map_rgb_alpha_color */
-  gx_page_device_get_page_device,
-  NULL,				/* get_alpha_bits */
-  NULL,				/* copy_alpha */
-  NULL,				/* get_band */
-  NULL,				/* copy_rop */
-  gdev_pdf_fill_path,
-  gdev_pdf_stroke_path,
-  gdev_pdf_fill_mask,
-  NULL,				/* fill_trapezoid */
-  NULL,				/* fill_parallelogram */
-  NULL,				/* fill_triangle */
-  NULL,				/* draw_thin_line */
-  NULL,				/* begin_image */
-  NULL,				/* image_data */
-  NULL,				/* end_image */
-  gdev_pdf_strip_tile_rectangle,
-  NULL,				/* strip_copy_rop */
-  NULL,				/* get_clipping_box */
-  gdev_pdf_begin_typed_image,
-  psdf_get_bits_rectangle,	/* get_bits_rectangle */
-  NULL,				/* map_color_rgb_alpha */
-  gdev_pdf_create_compositor,	/* create_compositor */
-  NULL,				/* get_hardware_params */
-  gdev_pdf_text_begin,
-  NULL,				/* finish_copydevice */
-  gdev_pdf_begin_transparency_group,	/* begin_transparency_group */
-  gdev_pdf_end_transparency_group,	/* end_transparency_group */
-  gdev_pdf_begin_transparency_mask,	/* begin_transparency_mask */
-  gdev_pdf_end_transparency_mask,	/* end_transparency_mask */
-  NULL,	            /* discard_transparency_layer */
-  NULL,				/* get_color_mapping_procs */
-  NULL,				/* get_color_comp_index */
-  NULL,				/* encode_color */
-  NULL,				/* decode_color */
-  NULL, 			/* pattern_manage */
-  gdev_pdf_fill_rectangle_hl_color, 	/* fill_rectangle_hl_color */
-  gdev_pdf_include_color_space, 	/* include_color_space */
-  NULL,				/* fill_linear_color_scanline */
-  NULL,				/* fill_linear_color_trapezoid */
-  NULL,				/* fill_linear_color_triangle */
-  NULL,				/* update_spot_equivalent_colors */
-  NULL,				/* ret_devn_params */
-  gdev_pdf_fillpage,		/* fillpage */
-  NULL,				/* push_transparency_state */
-  NULL,				/* pop_transparency_state */
-  NULL,				/* put_image */
-  gdev_pdf_dev_spec_op,		/* dev_spec_op */
-  NULL,             /* copy_planes */
-  NULL,             /* get_profile */
-  NULL,             /* set_graphics_type_tag */
-  NULL,             /* strip_copy_rop2 */
-  NULL,             /* strip_tile_rect_devn */
-  NULL,             /* copy_alpha_hl_color */
-  NULL,             /* process_page */
-  NULL,             /* transform_pixel_region */
-  gdev_pdf_fill_stroke_path /* fill_stroke_path */
- },
+ { 0 },
  psdf_initial_values(PSDF_VERSION_INITIAL, 0 /*false */ ),  /* (!ASCII85EncodePages) */
  0,                     /* pdf_font_dir */
  PDF_FOR_OPDFREAD,		/* is_ps2write */
@@ -174,6 +100,7 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  0 /*false*/,			/* fill_overprint */
  0 /*false*/,			/* stroke_overprint */
  1 /* Absolute Colorimetric */, /* rendering intent */
+ 0 /*false */,                  /* user_icc */
  0 /*false*/,                   /* remap_fill_coilor */
  0 /*false*/,                   /* remap_stroke_coilor */
  gs_no_id,			/* halftone_id */
@@ -186,7 +113,11 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  {{0}},				/* xref */
  {{0}},				/* asides */
  {{0}},				/* streams */
- {{0}},				/* pictures */
+ {{0}},				/* ObjStm */
+ gs_no_id,          /* ObjStm_id */
+ NULL,              /* ObjStmOffsets */
+ 0,                 /* NumObjstmObjects */
+ true,             /* doubleXref */
  0,				/* next_id */
  0,				/* Catalog */
  0,				/* Info */
@@ -258,6 +189,7 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  {{0, 0}, {0, 0}},  /* Charproc BBox */
  0,				/* charproc_just_accumulated */
  1,             /* PS_accumulator */
+ 0,             /* Scaled_accumulator */
  0,				/* accumulating_a_global_object */
  0,				/* cgp */
  0,				/* substituted_pattern_count */
@@ -309,7 +241,16 @@ const gx_device_pdf PDF_DEVICE_IDENT =
  NULL,                  /* OCRUnicode */
  0,                     /* OCR_char_code */
  0,                     /* OCR_glyph */
- NULL                   /* ocr_glyphs */
+ NULL,                  /* ocr_glyphs */
+ 0,                     /* initial_pattern_state */
+ false,                 /* OmitInfoDate */
+ false,                 /* OmitXMP */
+ false,                 /* OmitID */
+ false,                 /* ModifiesPageSize */
+ false,                 /* ModifiesPageOrder */
+ true,                  /* WriteXRefStm */
+ true,                  /* WriteObjStms */
+ 0                      /* PendingOC */
 };
 
 #else
