@@ -1,4 +1,4 @@
-/* Copyright (C) 2019-2021 Artifex Software, Inc.
+/* Copyright (C) 2019-2023 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* Routines for dealing with devices */
@@ -103,6 +103,24 @@ int pdfi_device_set_param_bool(gx_device *dev, const char *param, bool value)
     return code;
 }
 
+int pdfi_device_set_param_float(gx_device *dev, const char *param, float value)
+{
+    int code;
+    gs_c_param_list list;
+    float paramval = value;
+
+    gs_c_param_list_write(&list, dev->memory);
+
+    code = param_write_float((gs_param_list *)&list, param, &paramval);
+    if (code < 0) goto exit;
+    gs_c_param_list_read(&list);
+    code = gs_putdeviceparams(dev, (gs_param_list *)&list);
+
+ exit:
+    gs_c_param_list_release(&list);
+    return code;
+}
+
 /* Checks whether a parameter exists for the device */
 bool pdfi_device_check_param_exists(gx_device *dev, const char *param)
 {
@@ -134,13 +152,19 @@ void pdfi_device_set_flags(pdf_context *ctx)
     ctx->device_state.preserve_tr_mode = pdfi_device_check_param_bool(dev, "PreserveTrMode");
     ctx->device_state.preserve_smask = pdfi_device_check_param_bool(dev, "PreserveSMask");
     ctx->device_state.HighLevelDevice = pdfi_device_check_param_bool(dev, "HighLevelDevice");
+    ctx->device_state.ForOPDFRead = pdfi_device_check_param_bool(dev, "ForOPDFRead");
+    ctx->device_state.WantsPageLabels = pdfi_device_check_param_bool(dev, "WantsPageLabels");
+    ctx->device_state.PassUserUnit = pdfi_device_check_param_bool(dev, "PassUserUnit");
 
     /* See if it is a DeviceN (spot capable) */
     ctx->device_state.spot_capable = dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0);
 
+    ctx->device_state.ModifiesPageSize = pdfi_device_check_param_bool(dev, "ModifiesPageSize");
+    ctx->device_state.ModifiesPageOrder = pdfi_device_check_param_bool(dev, "ModifiesPageOrder");
+
     /* If multi-page output, can't do certain pdfmarks */
     if (ctx->device_state.writepdfmarks) {
-        if (gx_outputfile_is_separate_pages(((gx_device_vector *)dev)->fname, dev->memory)) {
+        if (gx_outputfile_is_separate_pages(((gx_device_vector *)dev)->fname, dev->memory) || ctx->device_state.ModifiesPageOrder) {
             ctx->args.no_pdfmark_outlines = true;
             ctx->args.no_pdfmark_dests = true;
         }
