@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
-   CA 94945, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  39 Mesa Street, Suite 108A, San Francisco,
+   CA 94129, USA, for further information.
 */
 
 /* xref parsing */
@@ -108,7 +108,7 @@ static int read_xref_stream_entries(pdf_context *ctx, pdf_c_stream *s, int64_t f
         }
 
         entry = &ctx->xref_table->xref[i];
-        if (entry->object_num != 0)
+        if (entry->object_num != 0 && !entry->free)
             continue;
 
         entry->compressed = false;
@@ -1132,7 +1132,7 @@ int pdfi_read_xref(pdf_context *ctx)
 
 repair:
     (void)pdfi_loop_detector_cleartomark(ctx);
-    if (!ctx->repaired)
+    if (!ctx->repaired && !ctx->args.pdfstoponerror)
         return(pdfi_repair_file(ctx));
     return 0;
 }
